#include "console.h"

namespace yx
{
	namespace IO
	{
		int open_strict(std::string filename, int type)
		{
			const char* fn = filename.c_str();
			int fd = open(fn, type, 0x0180);
			if(fd == -1)
			{
				fprintf(stderr, "Could not open file:[%s], Error Code:[%d]\n", fn, errno);
				fprintf(stderr, "Progress Abort.\n");
				exit( E_ABORT_IO );
			}
			return fd;
		}
		
		int write_strict(int fd, unsigned char* buffer, long len)
		{
			int rlt = write(fd, buffer, len);
			if(rlt == -1)
			{
				fprintf(stderr, "Failed to write to fd:[%d], Error Code:[%d]\n", fd, len);
				fprintf(stderr, "Progress Abort.\n");
				exit( E_ABORT_IO );
			}
			return rlt;
		}
	}
	
	namespace File
	{
		std::string get_extention(std::string filename)
		{
			return filename.substr(filename.find_last_of('.') + 1);
		}
		
		std::string get_prefix(std::string filename)
		{
			return filename.substr(0, filename.find_last_of('.'));
		}
		
		std::string get_path(std::string filename, char ps)
		{
			return filename.substr(0, filename.find_last_of(ps));
		}
	}
	
	namespace String
	{
		std::string itos(int value)
		{
			std::string result;
			while(value)
			{
				result = (char)((value % 10) - '0') + result;
				value /= 10;
			}
			return result;
		}
	}
	
	namespace gra
	{
		void progress_bar(int prog, int total)
		{
			printf("\r%3d%% - [", prog * 100 / total);
			for(int i = 1; i <= 25; ++i)
			{
				if(i <= prog * 25 / total) putchar('#');
				else putchar(' ');
			}
			printf("]%d/%d", prog, total);
		}
	}
}
