#include "lzr.h"


#define LZ_METHOD_ORIG 0
#define LZ_METHOD_CPYN 1
#define LZ_METHOD_RPTN 2
#define LZ_METHOD_DICT 3

struct flagMgr
{
	std::vector<unsigned char> data;
	unsigned char cache;
	unsigned int  pos;
	unsigned long bits;

	flagMgr() : cache(0), pos(0), bits(-1)
	{
	}

	void add_flag(unsigned char in)
	{
		cache |= (in & 0x3) << (6 - pos);
		if(6 < (pos = pos + 2)) data.push_back(cache), cache = 0, pos = 0;
		//else pos += 2;
	}

	std::basic_string<unsigned char> get_flag()
	{
		std::basic_string<unsigned char> flg(data.begin(), data.end());
		if(pos)
		{
			flg += cache;
		}
//		bits = data.size() * 8 + pos;
		return flg;
	}

	int get_bits()
	{
		return bits = data.size() * 8 + pos;
	}
};

void save_hangup(std::queue<unsigned char> & hangup, flagMgr * flag, std::basic_string<unsigned char> & out)//checked, work well 
{
	if(hangup.size() > 4)
	{
		if(hangup.size() > 0x7F)   //out of range
		{
			while(hangup.size() > 0x7f)   //store the part the out of range
			{
				int n = 0x7f;
				flag->add_flag(LZ_METHOD_CPYN);
				out += (unsigned char)n;
				while(n--)
				{
					out += hangup.front();
					hangup.pop();
				}
			}
			if(hangup.size() > 4)
			{
				flag->add_flag(LZ_METHOD_CPYN);
				out += (byte_t)hangup.size();
				while(!hangup.empty())
				{
					out += hangup.front();
					hangup.pop();
				}
			}
			else
			{
				while(!hangup.empty())
				{
					flag->add_flag(LZ_METHOD_ORIG);
					out += hangup.front();
					hangup.pop();
				}
			}//ALL saved
		}//end of out of range
		else
		{
			flag->add_flag(LZ_METHOD_CPYN);
			out += (byte_t)hangup.size();
			while(!hangup.empty())
			{
				out += hangup.front();
				hangup.pop();
			}
		}//ALL saved
	}
	else
	{
		while(!hangup.empty())
		{
			flag->add_flag(LZ_METHOD_ORIG);
			out += hangup.front();
			hangup.pop();
		}
	}
}

struct SymbDesc{
	U16 p:12;
	U16 n:4;
};

SymbDesc seek_symb(byte_t *window, byte_t *buffer, byte_t *end);

long lzr(byte_t *data, long length, byte_t ** result)
{
	printf("lzr(): get data, length:%ld\n", length);
	bool flag_end = false;
	
	std::basic_string<byte_t> lzdata;
	std::queue<byte_t> hangup;
	flagMgr fm;
	
	byte_t *LZ_WINDOW   = data - LZ_WINDOW_SIZE;  //set window pointer
	byte_t *LZ_BUFFER   = data;                   //set buffer size
	byte_t *LZ_DATA_END = data + length;          //it marks the termination, shouldn't be accessed!
	
	#define LZ_BUFFER_END (LZ_BUFFER + LZ_BUFFER_SIZE)
	//#define REPO if(!((int)LZ_BUFFER & 0xFF)) {printf("\rProgress: %06lld/%06ldbyte(s) compressed", LZ_BUFFER - data, length);}else{}
	long long last_repo = clock();
	
	while(LZ_WINDOW < data)//below phase: the window start at data, not LZ_WINDOW
	{
		if(clock() - last_repo >= REPO_GAP) printf("\rProgress: %06lld/%06ldbyte(s) compressed", LZ_BUFFER - data, length), last_repo = clock();
		
		if(LZ_BUFFER == LZ_DATA_END) { flag_end = true; break; }//if process is over
		if(LZ_BUFFER[0] == LZ_BUFFER[1] && LZ_BUFFER[1] == LZ_BUFFER[2])//if find many repeat
		{
			save_hangup(hangup, &fm, lzdata);
			byte_t st = *LZ_BUFFER;//set repeat symbol
			I8 rpt_cnt = 0;        //to count repeats
			while(st == *LZ_BUFFER && LZ_BUFFER < LZ_DATA_END && rpt_cnt < 0x7f)//0x7f:I8 upper limit
			{
				++rpt_cnt, ++LZ_BUFFER, ++LZ_WINDOW;
			}
			lzdata += (byte_t) rpt_cnt;   //append repeat account(n)
			lzdata += st;                 //append data
			fm.add_flag(LZ_METHOD_RPTN);  //set flag
			continue;
		}
		SymbDesc sdesc = seek_symb(data, LZ_BUFFER, MIN(LZ_BUFFER_END, LZ_DATA_END));//find symbol that could match
		if(sdesc.n > LZ_IGNORE_LENGTH)
		{
			debug("dict: p:%d, n:%d\n", sdesc.p, sdesc.n);
			save_hangup(hangup, &fm, lzdata); //save symbol(s) that was hung up before
			fm.add_flag(LZ_METHOD_DICT);
			byte_t temp = sdesc.p >> 4;       //build double-byte forward pointer(12 bits) et length(4 bits) 
			lzdata += temp;
			temp = (sdesc.p & 0xF) << 4 | (sdesc.n & 0xF);
			lzdata += temp;
			LZ_BUFFER += sdesc.n, LZ_WINDOW += sdesc.n;//move on
		}
		else
		{
			debug("ori:%c\n", *LZ_BUFFER);
			hangup.push(*LZ_BUFFER);//temporary hang up for more information to analyze the strategy
			++LZ_BUFFER, ++LZ_WINDOW;
		}
	}
	if(!flag_end) while(LZ_BUFFER < LZ_DATA_END)//if still has unprocessed data, process until attatch the ending
	{
		if(clock() - last_repo >= REPO_GAP) printf("\rProgress: %06lld/%06ldbyte(s) compressed", LZ_BUFFER - data, length), last_repo = clock();
		
		if(LZ_BUFFER == LZ_DATA_END) { flag_end = true; break; }//if process is over
		if(LZ_BUFFER[0] == LZ_BUFFER[1] && LZ_BUFFER[1] == LZ_BUFFER[2])//if find many repeat
		{
			save_hangup(hangup, &fm, lzdata);
			byte_t st = *LZ_BUFFER;//set repeat symbol
			I8 rpt_cnt = 0;        //to count repeats
			while(st == *LZ_BUFFER && LZ_BUFFER < LZ_DATA_END && rpt_cnt < 0x7f)//0x7f:I8 upper limit
			{
				++rpt_cnt, ++LZ_BUFFER, ++LZ_WINDOW;
			}
			lzdata += (byte_t) rpt_cnt;   //append repeat account(n)
			lzdata += st;                 //append data
			fm.add_flag(LZ_METHOD_RPTN);  //set flag
			continue;
		}
		SymbDesc sdesc = seek_symb(LZ_WINDOW , LZ_BUFFER, MIN(LZ_BUFFER_END, LZ_DATA_END));//find symbol that could match
		if(sdesc.n > LZ_IGNORE_LENGTH)
		{
			debug("dict: p:%d, n:%d\n", sdesc.p, sdesc.n);
			save_hangup(hangup, &fm, lzdata); //save symbol(s) that was hung up before
			fm.add_flag(LZ_METHOD_DICT);
			byte_t temp = sdesc.p >> 4;       //build double-byte forward pointer(12 bits) et length(4 bits) 
			lzdata += temp;
			temp = (sdesc.p & 0xF) << 4 | (sdesc.n & 0xF);
			lzdata += temp;
			LZ_BUFFER += sdesc.n, LZ_WINDOW += sdesc.n;//move on
		}
		else
		{
			debug("ori:%c\n", *LZ_BUFFER);
			hangup.push(*LZ_BUFFER);//temporary hang up for more information to analyze the strategy
			++LZ_BUFFER, ++LZ_WINDOW;
		}
	}
	if(!hangup.empty()) save_hangup(hangup, &fm, lzdata);
	
	LZRHD hd;
	hd.data_leng = lzdata.size();
	hd.flag_bits = fm.get_bits();
	hd.orig_leng = length;
	memcpy(hd.symb, "LZR", sizeof("LZR"));
	
	int flag_leng = (hd.flag_bits + 7)/8;
	
	byte_t *rst = new byte_t[sizeof(LZRHD) + flag_leng + hd.data_leng];
	memcpy(rst, &hd, sizeof(LZRHD));
	memcpy(rst + sizeof(LZRHD), fm.get_flag().c_str(), flag_leng);
	memcpy(rst + sizeof(LZRHD) + flag_leng, lzdata.c_str(), lzdata.size());
	
	*result = rst;
	
	return sizeof(LZRHD) + (hd.flag_bits + 7)/8 + hd.data_leng;
	
	#undef LZ_BUFFER_END 
	#undef REPO
}

SymbDesc seek_symb(byte_t *window, byte_t *buffer, byte_t *end)//checked ,work well
{
	int il = end - buffer, jl = buffer - window;
	
	U8 cmp_ary[LZ_BUFFER_SIZE][LZ_WINDOW_SIZE];
	for(int i = 0; i < jl; ++i)
		if(window[i] == *buffer) cmp_ary[0][i] = 1;
		else cmp_ary[0][i] = 0;
	
	U16 lest = 0, y = 0;
	
	for(int i = 1; i < il; ++i)
	for(int j = i; j < jl; ++j)
		if(cmp_ary[i - 1][j - 1] && window[j] == buffer[i])
			cmp_ary[i][j] = cmp_ary[i - 1][j - 1] + 1, lest = (lest <= cmp_ary[i][j])?((y = j - cmp_ary[i][j] + 1), cmp_ary[i][j]):lest;
			//let it count match length,      let it record the deapest length, if need update, y record the begining loc
		else cmp_ary[i][j] = 0;
	
	SymbDesc rlt;
	if(!lest) rlt.n = 0, rlt.p = 0;
	else rlt.p = buffer - (y + window), rlt.n = lest;
	return rlt;
	//excepted O(n*m)
}

//dustbin
/*
		{
			cmp_ary[i][j] = cmp_ary[i - 1][j - 1] + 1;
			if(lest < cmp_ary[i][j])
			{
				y = j - cmp_ary[i][j] + 1;
				lest = cmp_ary[i][j];
			}
		}
*/
