/*
	Name: Lzr Tool v1.0 2019/9/3 13:45
	Copyright:
	Author: YanXi
	Contact: 1197129380 [at] qq.com
	Date: 05-09-19 13:44
	Description: A tool to (un)compresse lzr compressed data used by Gust
*/

#include "msg.h"
#include "lzr.h"
#include "unlzr.h"
#include "common.h"

//#include <queue>
//#include <vector>

static const char * MSG_Usage = MSG_USG;
//static bool         quiet     = false;

int main(int argc, char ** argv)
{
	std::string input_filename, output_filename;
	int mode = 0, depth_limit = 16;
	if(argc == 1)
	{
		printf("Lzr Tool ver.1.1 coded by Yan Xi\n==================\n");
		printf( MSG_Usage );
		return 0;
	}
	else if(argc == 2)
	{
		input_filename  = argv[1];
		if(yx::File::get_extention(input_filename) != std::string("lzr"))
		{
//			fprintf(stderr, "File Extention Error!\n");
			output_filename = input_filename + ".lzr";
			printf("Mode:[Compress]\n");
			mode = M_C;
		}
		else
		{
			printf("Mode:[Extract]\n");
			output_filename = yx::File::get_prefix(input_filename);
			mode = M_UC;
		}
	}
	else
	{
		for(int _I = 1; _I <= argc; ++_I)
		{
			if(argv[_I][0] != '-')
			{
				fprintf(stderr, "Invalid param!\n");
				exit(E_INVALID_IN);
			}
			switch(argv[_I][1])
			{
				case 'i':
					input_filename = argv[ ++_I ];
					break;
				case 'o':
					output_filename = argv[ ++_I ];
					break;
				case 'd':
					sscanf(argv[ ++_I ], "%d", &depth_limit);
					break;
				case 'e':
					mode = M_UC;
					printf("Mode:[Extract]\n");
					break;
				case 'c':
					mode = M_C;
					printf("Mode:[Compress]\n");
					break;
				case 'q':
					freopen("nul", "w", stdout);
					break;
				default:
					fprintf(stderr, "Invalid param!\n");
					exit(E_INVALID_IN);
			}
		}
		if(output_filename.empty())
		{
			if(mode == M_UC && (yx::File::get_extention(input_filename) == std::string(".lzr")))
				output_filename = yx::File::get_prefix(input_filename);
			else output_filename = input_filename + ".lzr";
		}
	}//cmd input proccess section over

	printf("Input filename:[%s]\nOutput filename:[%s]\n", input_filename.c_str(), output_filename.c_str());

	int ipf = yx::IO::open_strict(input_filename, O_RDONLY | O_BINARY),
	    opf = yx::IO::open_strict(output_filename, O_CREAT | O_WRONLY | O_BINARY | O_TRUNC);

	if(mode == M_UC)   //uncompress
	{
		LZRHD              lzrh;
		unsigned char*     out      = NULL;

		read(ipf, &lzrh, sizeof(lzrh));

		out  = new unsigned char[lzrh.orig_leng];
		unsigned char* lzrdata  = new unsigned char[lzrh.data_leng];
		unsigned long  flag_len = (lzrh.flag_bits + 7) / 8;
		unsigned char* lzrflag  = new unsigned char[flag_len];

		read(ipf, lzrflag, flag_len);
		read(ipf, lzrdata, lzrh.data_leng);

		close( ipf );

		if(strcmp((char*)lzrh.symb, "LZR"))
		{
			fprintf(stderr, "Discorrect file format!\n");
			exit(E_INVALID_IN);
		}
		printf("flag bits:%lu data length:%lu original length:%lu\n\n", lzrh.flag_bits, lzrh.data_leng, lzrh.orig_leng);

		unlzr(&lzrh,
		      lzrflag,
		      lzrdata,
		      out);

		yx::IO::write_strict(opf, out, lzrh.orig_leng);
		close(opf);
	}
	else
	{
		unsigned long long file_len = filelength( ipf );
		unsigned char*     orig_dat = new unsigned char[file_len];
		read(ipf, orig_dat, file_len);
		close(ipf);

		byte_t *rstdata;
		int length = lzr(orig_dat, file_len, &rstdata);
		printf("\nCompress complete! Result length: %d", length);
		yx::IO::write_strict(opf, rstdata, length);
		close(opf);
	}
	printf("\nTime used: %lld(s)", clock()/CLOCKS_PER_SEC);
	return 0;
}

